<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 *  @author    SEBPAYMENTGATEWAY
 *  @copyright since 2015 SEBPAYMENTGATEWAY
 *  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class AccountInfo
{
    /**
     * @var array
     */
    private $info;

    /**
     * @var array
     */
    private $creditCardInfo;

    /**
     * @var array
     */
    private $payPalInfo;

    /**
     * @var array[]
     */
    private $bankInfo;

    /**
     * AccountInfo constructor.
     */
    public function __construct() {
        $this->bankInfo = [
            'EE' => [],
            'LV' => [],
            'LT' => [],
        ];

        $this->populatePaymentInfo();
    }

    /**
     * Get the account info.
     *
     * @return array|mixed
     */
    public function getAccountInfo()
    {
        if (!empty($this->info)) {
            return $this->info;
        }

        $isLiveMode = Configuration::get('SEBPAYMENTGATEWAY_LIVE_MODE');
        $apiUsername = $isLiveMode ? Configuration::get('SEBPAYMENTGATEWAY_API_USERNAME') : Configuration::get('SEBPAYMENTGATEWAY_TEST_API_USERNAME');
        $apiSecret = $isLiveMode ? Configuration::get('SEBPAYMENTGATEWAY_SHARED_SECRET') : Configuration::get('SEBPAYMENTGATEWAY_TEST_SHARED_SECRET');
        $accountInfoBaseUrl = $isLiveMode ? 'https://payment.ecommerce.sebgroup.com/api/v3/processing_accounts' : 'https://igw-demo.every-pay.com/api/v3/processing_accounts';
        $accountInfoUrl = $this->getPaymentAccountInfoUrl($accountInfoBaseUrl, Configuration::get('SEBPAYMENTGATEWAY_ACCOUNT_ID'), $apiUsername);
        $this->info = json_decode($this->getApiAccountInfo($accountInfoUrl, $apiUsername, $apiSecret), true);

        return $this->info;
    }

    /**
     * Get the account info from the API.
     *
     * @param string $url
     * @param string $username
     * @param string $password
     *
     * @return bool|string
     */
    private function getApiAccountInfo($url, $username, $password)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json'
            )
        );
        curl_setopt($ch, CURLOPT_USERPWD, $username . ':' . $password);
        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }

    /**
     * Get the bank information.
     *
     * @return array[]
     */
    public function getBankInfo()
    {
        return $this->bankInfo;
    }

    /**
     * Get the credit card information.
     *
     * @return array
     */
    public function getCreditCardInfo()
    {
        return $this->creditCardInfo;
    }

    /**
     * Get the PayPal information.
     *
     * @return array
     */
    public function getPayPalInfo()
    {
        return $this->payPalInfo;
    }

    /**
     * Get the API payment account info URL.
     *
     * @param string $baseUrl
     * @param string $accountName
     * @param string $apiUsername
     *
     * @return string
     */
    private function getPaymentAccountInfoUrl($baseUrl, $accountName, $apiUsername)
    {
        return $baseUrl . '/' . $accountName . '?api_username=' . $apiUsername;
    }

    /**
     * Populate the bank information.
     *
     * @param array $method
     *
     * @return void
     */
    private function populateBankInfo($method) {
        if (!empty($method['country_code'])) {
            $this->bankInfo[$method['country_code']][] = $method;
        } else {
            foreach ($this->bankInfo as $bank) {
                $bank[] = $method;
            }
        }
    }

    /**
     * Populate the payment information.
     *
     * @return void
     */
    public function populatePaymentInfo()
    {
        $this->getAccountInfo();

        foreach ($this->info['payment_methods'] as $method) {
            if ($method['source'] === 'card') {
                $this->creditCardInfo = $method;
            } elseif ($method['source'] === 'paypal') {
                $this->payPalInfo = $method;
            } else {
                $this->populateBankInfo($method);
            }
        }
    }
}
