<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 *  @author    EveryPay
 *  @copyright since 2015 EveryPay
 *  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class SebPaymentGateway extends PaymentModule
{
    const ORDER_BEFORE = 'before';
    const ORDER_AFTER = 'after';

    const PAYMENT_IFRAME = 'iframe';
    const PAYMENT_REDIRECT = 'redirect';

    const APPLE_BUTTON_WHITE = 'white';
    const APPLE_BUTTON_WHITE_OUTLINE = 'white_with_outline';
    const APPLE_BUTTON_BLACK = 'black';
    const RETURN_SUCCESS = 'completed';
    const RETURN_CANCEL = 'voided';
    const RETURN_FAILED = 'failed';
    const RETURN_SETTLED = 'settled';
    const RETURN_PROCESSING = 'sent_for_processing';
    const RETURN_INVALID_AMOUNT = 'invalid_amount';

    const PAYMENT_OPTION_CARD = 0;
    const PAYMENT_OPTION_BANKLINK = 1;
    const PAYMENT_OPTION_PAYPAL = 2;
    const PAYMENT_OPTION_APPLE_PAY = 3;

    const PAYMENT_OPTION_CHECKBOX_NAME_BASE = 'enabled_payment_option';

    const DEBUG_LOGGING = 0;

    const PAYMENT_STATUS_TABLE = 'seb_payment_gateway_payment_status';

    public $os_processing = array();
    public $os_awaiting = array();

    /**
     * @var AccountInfo
     */
    public $accountInfo;

    const DOC_VERSION = '1.1';

    public function __construct()
    {
        $this->name = 'sebpaymentgateway';
        $this->tab = 'payments_gateways';
        $this->author = 'EveryPay';
        $this->version = '7.4.3';
        $this->is_eu_compatible = 1;
        $this->bootstrap = true;
        $this->controllers = array('mycards');
        $this->displayName = $this->l('SEB payment gateway');
        $this->description = $this->l('Accept payments via SEB payment gateway solution.');
        $this->confirmUninstall = $this->l('Are you sure to uninstall SEB payment gateway?');
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);
        $this->module_key = 'db60dd5f7405f17e07daff841db0bb14';
        $this->os_processing = array(
            'status_name' => 'PS_OS_SEBPAYMENTGATEWAY_PROCESSING',
            'description' => sprintf('Sent for processing (%s)', $this->displayName),
            'descriptionEE' => sprintf('Makse töötlemisel (%s)', $this->displayName),
            'color' => "#fde4ba"
        );
        $this->os_awaiting = array(
            'status_name' => 'PS_OS_SEBPAYMENTGATEWAY',
            'description' => sprintf('Awaiting %s payment', $this->displayName),
            'descriptionEE' => sprintf('Ootab %s maksmist', $this->displayName),
            'color' => "#fde4ba"
        );
        $this->loadAccountInfo();
        $this->accountInfo = new AccountInfo();
        parent::__construct();
    }

    public function install()
    {
        Configuration::updateValue($this->formatName('demo_url'), 'https://igw-demo.every-pay.com/api/v3/payments/');
        Configuration::updateValue($this->formatName('production_url'), 'https://payment.ecommerce.sebgroup.com/api/v3/payments/');
        Configuration::updateValue($this->formatName('apple_pay_v4_demo_url'), 'https://igw-demo.every-pay.com/api/v4/payments/');
        Configuration::updateValue($this->formatName('apple_pay_v4_production_url'), 'https://payment.ecommerce.sebgroup.com/api/v4/payments/');
        Configuration::updateValue($this->formatName('apple_pay_js_demo_url'), 'https://igw-demo.every-pay.com/apple_pay/apple-pay-client-1.4.js');
        Configuration::updateValue($this->formatName('apple_pay_js_production_url'), 'https://payment.ecommerce.sebgroup.com/apple_pay/apple-pay-client-1.4.js');
        Configuration::updateValue($this->formatName('create_order'), self::ORDER_BEFORE);
        Configuration::updateValue($this->formatName('payment_method'), self::PAYMENT_REDIRECT);
        Configuration::updateValue($this->formatName('apple_pay_button_style'), self::APPLE_BUTTON_BLACK);
        Configuration::updateValue($this->formatName($this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_CARD)), '1');
        Configuration::updateValue($this->formatName($this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_BANKLINK)), '1');

        $sql= "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."seb_payment_gateway_cards`(
        `id_card` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
        `card_no` VARCHAR(6) NOT NULL,
        `card_exp_year` VARCHAR(6) NOT NULL,
        `card_exp_month` VARCHAR(6) NOT NULL,
        `card_type` varchar(100) NOT NULL,
        `id_customer` int(11) NOT NULL,
        `token` VARCHAR(256) NOT NULL,
        `is_default` TINYINT(4) NOT NULL,
        `date_added` date NOT NULL
        )";

        Db::getInstance()->Execute($sql);
        $this->createSebPaymentGatewayPaymentsTable();
        $this->createPaymentStatusTable();

        return
            parent::install() &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayPaymentReturn') &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('customerAccount') &&
            $this->registerHook('displayMyAccountBlock') &&
            $this->registerHook('actionAdminControllerSetMedia') &&
            $this->registerHook('actionFrontControllerSetMedia') &&
            $this->addOrderState($this->os_processing) &&
            $this->addOrderState($this->os_awaiting);
    }

    public function createSebPaymentGatewayPaymentsTable()
    {
        $sql = "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."seb_payment_gateway_payments`(
        `order_reference` INT(11) NOT NULL,
        `payment_reference` VARCHAR(256) NOT NULL
        )";
        Db::getInstance()->execute($sql);
    }

    public function createPaymentStatusTable()
    {
        $sql = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . self::PAYMENT_STATUS_TABLE . "`(
            `id_status` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `order_reference` VARCHAR(256) NOT NULL,
            `status` VARCHAR(256) NOT NULL,
            INDEX `order_reference_index` (`order_reference`)
        )";
        Db::getInstance()->execute($sql);
    }

    public function uninstall()
    {
        Configuration::deleteByName($this->formatName('enabled_payment'));
        return parent::uninstall();
    }

    public function getContent()
    {
        $output = '';
        $output .= $this->displayInfo();
        $output .= $this->postProcess();
        $output .= $this->displayForm();
        return $output;
    }

    protected function postProcess()
    {
        if (Tools::isSubmit('submitAddconfiguration')) {
            Configuration::updateValue($this->formatName('test_api_username'), Tools::getValue('test_api_username'));
            Configuration::updateValue($this->formatName('test_shared_secret'), Tools::getValue('test_shared_secret'));
            Configuration::updateValue($this->formatName('api_username'), Tools::getValue('api_username'));
            Configuration::updateValue($this->formatName('account_id'), Tools::getValue('account_id'));
            Configuration::updateValue($this->formatName('shared_secret'), Tools::getValue('shared_secret'));
            Configuration::updateValue($this->formatName('create_order'), Tools::getValue('create_order'));
            Configuration::updateValue($this->formatName('live_mode'), Tools::getValue('live_mode'));
            Configuration::updateValue($this->formatName('payment_method'), Tools::getValue('payment_method'));
            Configuration::updateValue($this->formatName('iframe_skin'), Tools::getValue('iframe_skin'));
            Configuration::updateValue($this->formatName('apple_pay_button_style'), Tools::getValue('apple_pay_button_style'));
            Configuration::updateValue($this->formatName('token_payment'), Tools::getValue('token_payment'));
            Configuration::updateValue($this->formatName('enabled_payment'), Tools::getValue('enabled_payment'));
            Configuration::updateValue($this->formatName('methods_order'), Tools::getValue('methods_order'));

            foreach ($this->getPaymentOptionsCheckboxNames() as $name) {
                Configuration::updateValue($this->formatName($name), (bool) Tools::getValue($name));
            }
        }
    }

    public function displayInfo()
    {
        $v = self::DOC_VERSION;
        $this->smarty->assign(array(
            'path' => $this->getPathUri(),
            'sebpaymentgateway_link' => 'https://support.ecommerce.sebgroup.com/',
            'sebpaymentgateway_doc' => 'https://support.ecommerce.sebgroup.com/merchant-support/prestashop-extension/',
        ));
        if (Tools::version_compare(_PS_VERSION_, '1.6', '>')) {
            return $this->fetch('module:sebpaymentgateway/views/templates/hook/info.tpl');
        } else {
            return $this->display(__FILE__, 'info_15.tpl');
        }
    }

    public function displayForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');
        $payment_method = array();
        $payment_method[] = array('value' => self::PAYMENT_IFRAME,
            'label' => $this->l('Embedded iFrame maksevorm'),
            'id' => 'payment_method_iframe');
        $payment_method[] = array('value' => self::PAYMENT_REDIRECT,
            'label' => $this->l('Edasisuunamine (redirect) makselehele'),
            'id' => 'payment_method_redirect');
        $apple_pay_button_styles = array();
        $apple_pay_button_styles[] = array(
            'value' => self::APPLE_BUTTON_WHITE,
            'label' => $this->l('White'),
            'id' => 'white'
        );
        $apple_pay_button_styles[] = array(
            'value' => self::APPLE_BUTTON_WHITE_OUTLINE,
            'label' => $this->l('White with outline'),
            'id' => 'white_with_outline'
        );
        $apple_pay_button_styles[] = array(
            'value' => self::APPLE_BUTTON_BLACK,
            'label' => $this->l('Black'),
            'id' => 'black'
        );
        $fields_form = array(array(
            'form' => array(
                'legend' => array('title' => $this->l('Settings')),
                'input' => array(
                    array(
                        'type'     => 'free',
                        'label'    => $this->l('Callback Notification URL'),
                        'name'     => 'free_string_name',
                        //'value' => 'test',
                    ),
                    array(
                        'type'     => 'radio',
                        'label'    => $this->l('Live Mode'),
                        'name'     => 'live_mode',
                        'required' => true,
                        'class'    => 't',
                        'values'   => array(
                            array('value' => 1, 'label' => $this->l('On'), 'id' => 'live_mode_on'),
                            array('value' => 0, 'label' => $this->l('Off'), 'id' => 'live_mode_off')
                        )
                    ),
                    array(
                        'type'     => 'radio',
                        'label'    => $this->l('Token payments'),
                        'name'     => 'token_payment',
                        'required' => true,
                        'class'    => 't',
                        'values'   => array(
                            array('value' => 1, 'label' => $this->l('On'), 'id' => 'token_payments_on'),
                            array('value' => 0, 'label' => $this->l('Off'), 'id' => 'token_payments_off')
                        )
                    ),
                    array(
                        'type'     => 'select',
                        'label'    => $this->l('Payment form type'),
                        'name'     => 'payment_method',
                        'required' => true,
                        'class'    => 't',
                        'options'   => array(
                            "query" => $payment_method,
                            "id"    => "value",
                            "name"  => "label"
                        )
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('iFrame skin'),
                        'name'     => 'iframe_skin',
                        'required' => false,
                        'class'    => 't',
                        'default'  => 'default',
                        'desc'     => $this->l('Appearance of the payment form can be customized in
                        SEB payment gateway Merchant Portal, under "Settings" > "iFrame skins".')
                    ),
                    array(
                        'type'     => 'select',
                        'label'    => $this->l('Apple Pay button style'),
                        'name'     => 'apple_pay_button_style',
                        'required' => true,
                        'class'    => 't',
                        'options'   => array(
                            "query" => $apple_pay_button_styles,
                            "id"    => "value",
                            "name"  => "label"
                        )
                    ),
                    array(
                        'type'     => 'radio',
                        'label'    => $this->l('Create order'),
                        'name'     => 'create_order',
                        'required' => true,
                        'class'    => 't',
                        'desc'     => $this->l('create_order_desc'),
                        'values'   => array(
                            array(
                                'value' => self::ORDER_BEFORE,
                                'label' => $this->l('Before payment'),
                                'id'    => 'sebpaymentgateway_order_before'
                            ),
                            array(
                                'value' => self::ORDER_AFTER,
                                'label' => $this->l('After payment'),
                                'id'    => 'sebpaymentgateway_order_after'
                            )
                        )
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Processing Account'),
                        'name'     => 'account_id',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('account_id_desc')
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('API Username'),
                        'name'     => 'api_username',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('api_username_desc')
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('API Secret'),
                        'name'     => 'shared_secret',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('shared_secret_desc'),
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Test API Username'),
                        'name'     => 'test_api_username',
                        'size'     => 50,
                        'desc'     => $this->l('Only used for testing payments in the demo environment.
                            You can find this in the SEB payment gateway demo merchant portal, under
                            “Settings/General settings/API username” (example: 49bcdd6fed983c12).'
                        )
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Test API Secret'),
                        'name'     => 'test_shared_secret',
                        'size'     => 50,
                        'desc'     => $this->l('Only used for testing payments in the demo environment.
                            You can find this in the SEB payment gateway demo merchant portal, under
                            “Settings/General settings/API secret” (example: e7ed4e55d5f73158f6cf2890fb1c950e).'
                        )
                    ),
                    array(
                        'type'     => 'checkbox',
                        'label'    => $this->l('Select payment types to use'),
                        'name'     => self::PAYMENT_OPTION_CHECKBOX_NAME_BASE,
                        'required' => true,
                        'class'    => 't',
                        'desc'     => $this->l('Select payment methods to be available in your shop and their
                            order on the checkout page.'
                        ),
                        'values'   => array(
                            'query' => $this->mapMethods(),
                            'id' => 'payment_option',
                            'name' => 'payment_option_name',
                        ),
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Methods Order'),
                        'name'     => 'methods_order',
                        'size'     => 50,
                        'desc'     => ''
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            )
        ));
        $helper_form = new HelperForm();
        $helper_form->module = $this;
        $helper_form->name_controller = $this->name;
        $helper_form->token = Tools::getAdminTokenLite('AdminModules');
        $helper_form->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper_form->default_form_language = $default_lang;
        $helper_form->allow_employee_form_lang = $default_lang;
        $helper_form->languages = Language::getLanguages(false);
        foreach ($helper_form->languages as $k => $language) {
            $is_default = (int)($language['id_lang'] == $helper_form->default_form_language);
            $helper_form->languages[$k]['is_default'] = $is_default;
        }
        $helper_form->fields_value['test_api_username'] = Configuration::get($this->formatName('test_api_username'));
        $helper_form->fields_value['test_shared_secret'] = Configuration::get($this->formatName('test_shared_secret'));
        $helper_form->fields_value['api_username'] = Configuration::get($this->formatName('api_username'));
        $helper_form->fields_value['account_id'] = Configuration::get($this->formatName('account_id'));
        $helper_form->fields_value['shared_secret'] = Configuration::get($this->formatName('shared_secret'));
        $helper_form->fields_value['create_order'] = Configuration::get($this->formatName('create_order'));
        $helper_form->fields_value['live_mode'] = Configuration::get($this->formatName('live_mode'));
        $helper_form->fields_value['token_payment'] = Configuration::get($this->formatName('token_payment'));
        $helper_form->fields_value['payment_method'] = Configuration::get($this->formatName('payment_method'));
        $helper_form->fields_value['iframe_skin'] = Configuration::get($this->formatName('iframe_skin'));
        $helper_form->fields_value['apple_pay_button_style'] = Configuration::get($this->formatName('apple_pay_button_style'));
        $helper_form->fields_value['enabled_payment'] = Configuration::get($this->formatName('enabled_payment'));
        $helper_form->fields_value['methods_order'] = $this->getSortOrder();

        $callbackInfo = $this->l('Ensure you enter the Callback Notification URL in the SEB payment gateway
            merchant portal under “Settings/E-shop Settings” to sync payment information between your online store
            and SEB payment gateway account.'
        );
        $callbackUrl = $this->context->link->getModuleLink($this->name,'return');

        $helper_form->fields_value['free_string_name'] = '<div style="padding-right: 20%; font-size: 12px ">'. $callbackInfo . '
        <br><br>
        Callback URL: <strong><i>'. $callbackUrl .'</i></strong></div>';



        foreach ($this->getPaymentOptionsCheckboxNames() as $name) {
            $helper_form->fields_value[$name] = Configuration::get($this->formatName($name));
        }

        $javascript = "<script>";
        $javascript .= "
        sebPaymentGatewayMethodCheck();
        $('#payment_method').attr('onchange', 'sebPaymentGatewayMethodCheck()');
        function sebPaymentGatewayMethodCheck(){
            if ($('#payment_method').val() === 'iframe'){
                $('#iframe_skin').parent().parent().show();
            }else{
                $('#iframe_skin').parent().parent().hide();
            }
        }
        ";
        $javascript .= "</script>";

        return $helper_form->generateForm($fields_form).$javascript;
    }

    private function getPaymentOptionsCheckboxNames()
    {
        return array(
            $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_CARD),
            $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_BANKLINK),
            $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_PAYPAL),
            $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_APPLE_PAY)
        );
    }

    private function getPaymentOptionCheckboxName($paymentOption)
    {
        return self::PAYMENT_OPTION_CHECKBOX_NAME_BASE . '_' . $paymentOption;
    }

    public function formatName($name = '')
    {
        return Tools::strtoupper($this->name.'_'.$name);
    }

    /**
     * Get the sort order for the payment methods.
     *
     * @return string
     */
    public function getSortOrder()
    {
        if (Configuration::get($this->formatName('methods_order'))) {
            return Configuration::get($this->formatName('methods_order'));
        } else {
            $sortOrder = '';
            $methodCount = count($this->mapMethods()) - 1;
            foreach($this->mapMethods() as $key => $value) {
                $sortOrder .= $value['payment_option'];

                if ($key !== $methodCount) {
                    $sortOrder .= ',';
                }
            }

            return $sortOrder;
        }
    }

    /**
     * Sort the payment methods at checkout based on the sort order provided.
     *
     * @param array $paymentMethods
     *
     * @return array
     */
    public function sortPaymentMethods($paymentMethods)
    {
        $sortOrder = explode(',', $this->getSortOrder());
        $sorted = [];

        foreach ($sortOrder as $order) {
            if (!empty($paymentMethods[$order])) {
                $sorted[$order] = $paymentMethods[$order];
                unset($paymentMethods[$order]);
            }
        }

        return array_merge($sorted, $paymentMethods);
    }

    /**
     * Map the payment methods for the checkbox
     *
     * @return array
     */
    public function mapMethods()
    {
        return [
            [
                'payment_option' => self::PAYMENT_OPTION_CARD,
                'payment_option_name' => $this->l('Card'),
            ],
            [
                'payment_option' => self::PAYMENT_OPTION_BANKLINK,
                'payment_option_name' => $this->l('Bank'),
            ],
            [
                'payment_option' => self::PAYMENT_OPTION_PAYPAL,
                'payment_option_name' => $this->l('PayPal'),
            ],
            [
                'payment_option' => self::PAYMENT_OPTION_APPLE_PAY,
                'payment_option_name' => $this->l('Apple Pay'),
            ]
        ];
    }

    /**
     * Check if the order was created before payment.
     *
     * @return bool
     */
    public function isOrderCreatedBefore()
    {
        return Configuration::get($this->formatName('create_order')) === self::ORDER_BEFORE;
    }

    public function getConfig($key)
    {
        return Configuration::get($this->formatName((string)$key));
    }

    public function getSebPaymentGatewayAPI()
    {
        if (!class_exists('SebPaymentGatewayLibrary')) {
            include $this->local_path.'lib/sebpaymentgateway_library.php';
        }

        $api = null;
        $live_mode = $this->getConfig('live_mode');

        $username = $this->getConfig((!$live_mode ? 'test_' : '').'api_username');
        $secret = $this->getConfig((!$live_mode ? 'test_' : '').'shared_secret');

        if (!empty($username) && !empty($secret)) {
            $api = new SebPaymentGatewayLibrary();
            $api->init($username, $secret);
        }

        return $api;
    }

    /**
     * Create Order State, coping mails and logo
     * @return boolean  true, if successful
     */
    public function addOrderState($stateArr)
    {
        $orderStateExist = false;
        $status_name = $stateArr['status_name'];
        $orderStateId = Configuration::get($status_name);
        $description = $stateArr['description'];
        $descriptionEE = $stateArr['descriptionEE'];

        if ($orderStateId) {
            $orderState = new OrderState($orderStateId);
            if ($orderState->id && !$orderState->deleted) {
                $orderStateExist = true;
            }
        } else {
            $query = 'SELECT os.`id_order_state` '.
                'FROM `%1$sorder_state_lang` osl '.
                'LEFT JOIN `%1$sorder_state` os '.
                'ON osl.`id_order_state`=os.`id_order_state` '.
                'WHERE osl.`name`="%2$s" OR osl.`name`="%3$s" AND os.`deleted`=0';

            $orderStateId =  Db::getInstance()->getValue(sprintf($query, _DB_PREFIX_, $description, $descriptionEE));
            if ($orderStateId) {
                Configuration::updateValue($status_name, $orderStateId);
                $orderStateExist = true;
            }
        }
        if (!$orderStateExist) {
            $languages = Language::getLanguages(false);
            $orderState = new OrderState();
            $estLangID = $this->getEstonianLangId() ?: false;

            foreach ($languages as $lang) {
                if ($lang['id_lang'] == $estLangID) {
                    $orderState->name[$lang['id_lang']] = $descriptionEE;
                } else {
                    $orderState->name[$lang['id_lang']] = $description;
                }
            }
            $orderState->send_email = 0;
            $orderState->invoice = 0;
            $orderState->color = $stateArr['color'];
            $orderState->unremovable = 0;
            $orderState->logable = 0;
            $orderState->delivery = 0;
            $orderState->hidden = 0;
            if ($orderState->add()) {
                Configuration::updateValue($status_name, $orderState->id);
                $orderStateExist = true;
            }
        }
        $file = $this->getLocalPath().'/views/img/sebpaymentgateway_status.gif';
        $newfile = _PS_IMG_DIR_.'os/' . $orderState->id . '.gif';
        copy($file, $newfile);

        return ($orderStateExist);
    }

    public function getOrderConfUrl($order)
    {
        return $this->context->link->getPageLink(
            'order-confirmation',
            true,
            $order->id_lang,
            array(
                'id_cart' => $order->id_cart,
                'id_module' => $this->id,
                'id_order' => $order->id,
                'key' => $order->secure_key
            )
        );
    }

    public function getOrderIdByReference($orderReference)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('orders');
        $sql->where('reference = '.'"'.$orderReference.'"');

        return Db::getInstance()->getValue($sql);
    }

    public function getOrderIdByCartId($cartId)
    {
        $sql = new DbQuery();
        $sql->select('id_order')
            ->from('orders')
            ->where('id_cart = '.(int)$cartId);

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Get the payment status based on the order reference.
     *
     * @param string $orderReference
     *
     * @return false|string|null
     */
    public function getPaymentStatusByReference($orderReference)
    {
        $sql = new DbQuery();
        $sql->select('status');
        $sql->from(self::PAYMENT_STATUS_TABLE);
        $sql->where('order_reference = ' . '"' . $orderReference . '"');

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Stores the payment status for an order.
     *
     * @param string $orderReference
     * @param string $paymentStatus
     *
     * @return void
     *
     * @throws PrestaShopDatabaseException
     */
    public function addOrderPaymentStatus($orderReference, $paymentStatus)
    {
        Db::getInstance()->insert(self::PAYMENT_STATUS_TABLE, [
            'order_reference' => pSQL($orderReference),
            'status' => pSQL($paymentStatus),
        ]);
    }

    /**
     * Updates the payment status for an order.
     *
     * @param string $orderReference
     * @param string $paymentStatus
     *
     * @return void
     */
    public function updateOrderPaymentStatus($orderReference, $paymentStatus)
    {
        \Db::getInstance()->update(
            self::PAYMENT_STATUS_TABLE,
            [
                'status' => pSQL($paymentStatus),
            ],
            'order_reference=' . pSQL($orderReference)
        );
    }

    //Hooks START
    public function hookDisplayHeader($params)
    {
        if (Tools::getValue('controller') == 'order') {
            $sebpaymentgatewayIdAddressDelivery = $this->context->cart->id_address_delivery;
            Media::addJsDef(['sebpaymentgatewayIdAddressDelivery' => $sebpaymentgatewayIdAddressDelivery]); // task 17971:
            // above passes delivery address id into JS, to get address country later
        }

        $this->context->controller->addCSS(($this->_path."views/css/").'sebpaymentgateway.css', 'all');
    }

    public function hookCustomerAccount($params)
    {
        $this->context->smarty->assign(array(
            'token_payment' => Configuration::get($this->formatName("token_payment")),
            'logged' => $this->context->customer->isLogged()
        ));
        return $this->fetch('module:sebpaymentgateway/views/templates/hook/sebpaymentgateway-myaccount.tpl');
    }

    public function hookDisplayMyAccountBlock($params)
    {
        return $this->hookCustomerAccount($params);
    }
    //Hooks END
    public function loadStorage()
    {
        if (!class_exists('CardStorageLibrary')) {
            include $this->local_path.'lib/CardStorageLibrary.php';
        }
    }
    private function loadAccountInfo()
    {
        if (!class_exists('AccountInfo')) {
            include $this->local_path . 'lib/AccountInfo.php';
        }
    }
    // New Payment Option set option visible on front pages START
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $cards = array();
        if ($this->context->customer->isLogged()) {
            $this->loadStorage();
            $id_customer = $this->context->customer->id;
            $cards = CardStorageLibrary::getCards($id_customer);
        }

        $live = Configuration::get($this->formatName('live_mode'));
        $enabled_payment = Configuration::get($this->formatName('enabled_payment'));
        $payment_method = Configuration::get($this->formatName('payment_method'));
        $bankLinkOption = new PaymentOption();
        $bankLinkOption
            ->setCallToActionText($this->l('Pay with Bank'))
            ->setModuleName($this->name);
        $paypalOption = new PaymentOption();
        $paypalOption
            ->setCallToActionText($this->l('Pay with PayPal'))
            ->setModuleName($this->name);
        $applePayOption = new PaymentOption();
        $applePayOption
            ->setCallToActionText($this->l('Pay with Apple Pay'))
            ->setBinary(true)
            ->setModuleName($this->name);

        $validationController = $this->context->link->getModuleLink('sebpaymentgateway', 'validation', array(), true);
        if ($payment_method == "iframe") {
            $this->context->smarty->assign("cart_id", $params['cart']->id);
            $skin = Configuration::get($this->formatName('iframe_skin'));
            if (empty($skin)) {
                $skin = "default";
            }

            $this->context->smarty->assign(array(
                'shop_url'=>__PS_BASE_URI__,
                'this_path' => $this->getPathUri(),
                'token_payment' => Configuration::get($this->formatName("token_payment")),
                'credit_card_logo' => $this->accountInfo->getCreditCardInfo()['logo_url'] ?? '',
                'paypal_logo' => $this->accountInfo->getPayPalInfo()['logo_url'] ?? '',
                'bank_links' => $this->accountInfo->getBankInfo(),
                'order_create_method' => Configuration::get($this->formatName("create_order")),
                'payment_link' => Configuration::get($this->formatName(($live == 1 ? "production_url" : "demo_url"))),
                'bank_payment_link' => $validationController,
                'cards' => $cards,
                'logged' => $this->context->customer->isLogged(),
                'apple_pay_payment_link' => Configuration::get($this->formatName(($live == 1 ? "apple_pay_v4_production_url" : "apple_pay_v4_demo_url"))),
                'store_name' => Configuration::get('PS_SHOP_NAME'),
                'apple_pay_js' => Configuration::get($this->formatName(($live == 1 ? "apple_pay_js_production_url" : "apple_pay_js_demo_url"))),
                'apple_pay_button_style' => Configuration::get($this->formatName('apple_pay_button_style')),
            ));

            $iframeCardOption = new PaymentOption();
            $iframeCardOption
                ->setCallToActionText($this->l('Pay with Card'))
                ->setModuleName($this->name)
                ->setForm($this->generateIframe());
            $bankLinkOption->setForm($this->generateBanklinks());
            $paypalOption
                ->setForm($this->generatePaypalIframe());
            $applePayOption->setForm($this->generateApplePay());

            //return $this->filterPayments($iframeCardOption, $bankLinkOption, $enabled_payment);
            return $this->filterPayments(array(
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_CARD) => $iframeCardOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_BANKLINK) => $bankLinkOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_PAYPAL) => $paypalOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_APPLE_PAY) => $applePayOption
            ));
        } else {
            $this->context->smarty->assign(array(
                'shop_url'=>__PS_BASE_URI__,
                'token_payment' => Configuration::get($this->formatName("token_payment")),
                'credit_card_logo' => $this->accountInfo->getCreditCardInfo()['logo_url'] ?? '',
                'paypal_logo' => $this->accountInfo->getPayPalInfo()['logo_url'] ?? '',
                'bank_links' => $this->accountInfo->getBankInfo(),
                'this_path' => $this->getPathUri(),
                'payment_link' => $validationController,
                'bank_payment_link' => $validationController,
                'cards' => $cards,
                'logged' => $this->context->customer->isLogged(),
                'apple_pay_payment_link' => Configuration::get($this->formatName(($live == 1 ? "apple_pay_v4_production_url" : "apple_pay_v4_demo_url"))),
                'store_name' => Configuration::get('PS_SHOP_NAME'),
                'apple_pay_js' => Configuration::get($this->formatName(($live == 1 ? "apple_pay_js_production_url" : "apple_pay_js_demo_url"))),
                'apple_pay_button_style' => Configuration::get($this->formatName('apple_pay_button_style')),
            ));
            $redirectOption = new PaymentOption();
            $redirectOption
                ->setCallToActionText($this->l('Pay with Card'))
                ->setModuleName($this->name)
                ->setForm($this->generateRedirect());
            $bankLinkOption->setForm($this->generateBanklinks());
            $paypalOption->setForm($this->generatePaypalRedirect());
            $applePayOption->setForm($this->generateApplePay());

            //return $this->filterPayments($redirectOption, $bankLinkOption, $enabled_payment);
            return $this->filterPayments(array(
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_CARD) => $redirectOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_BANKLINK) => $bankLinkOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_PAYPAL) => $paypalOption,
                $this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_APPLE_PAY) => $applePayOption
            ));
        }
    }
    // New Payment Option set option visible on front pages END

    public function filterPayments($paymentOptions)
    {
        $filteredPaymentOptions = array();
        foreach ($this->getPaymentOptionsCheckboxNames() as $name) {
            if (Configuration::get($this->formatName($name))) {
                if (!$this->isApplePayEnabled()
                    && $this->formatName($this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_APPLE_PAY)) === $this->formatName($name)) {
                    continue;
                }
                $filteredPaymentOptions[] = $paymentOptions[$name];
            }
        }

        return $this->sortPaymentMethods($filteredPaymentOptions);
    }

    private function isApplePayEnabled()
    {
        if (!Configuration::get($this->formatName($this->getPaymentOptionCheckboxName(self::PAYMENT_OPTION_CARD)))) {
            return false;
        }

        return true;
    }

    // Payment Option inside form - start
    public function generateIframe()
    {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/payment_iframe.tpl');
    }

    public function generateBanklinks() {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/payment_banklink.tpl');
    }

    public function generateRedirect()
    {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/payment_redirect.tpl');
    }

    public function generatePaypalRedirect()
    {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/paypal_redirect.tpl');
    }

    public function generatePaypalIframe()
    {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/paypal_iframe.tpl');
    }

    public function generateApplePay()
    {
        return $this->context->smarty->fetch('module:sebpaymentgateway/views/templates/hook/apple_pay.tpl');
    }

    // Payment Option inside form - end
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookDisplayPaymentReturn($params)
    {
        $order = $params['order'];
        if ($params['order']) {
            $this->smarty->assign(array(
                'shop_name' => $this->context->shop->name,
                'total' => Tools::displayPrice(
                    $params['order']->getOrdersTotalPaid()
                )
            ));
            $status = 'ok';
        } else {
            $status = 'error';
        }

        $orderHistory = $this->context->link->getPageLink(
            'order-detail',
            true,
            $order->id_lang,
            array(
                'id_order' => $order->id
            )
        );

        $this->smarty->assign(array(
            'status' => $status,
            'linkToOrder' => $orderHistory
        ));

        return $this->fetch('module:sebpaymentgateway/views/templates/hook/payment-return.tpl');
    }

    public function hookActionAdminControllerSetMedia()
    {
        if (Tools::getValue('controller') !== 'AdminModules') {
            return;
        }
        if (Tools::getValue('configure') !== $this->name) {
            return;
        }
        $this->context->controller->addCSS(
            $this->_path . 'views/css/sebpaymentgateway-admin.css'
        );
        $this->context->controller->addJS(
            'https://cdnjs.cloudflare.com/ajax/libs/Sortable/1.15.2/Sortable.min.js'
        );
        $this->context->controller->addJS(
            $this->_path . 'views/js/sebpaymentgateway-admin.js'
        );
    }

    public function hookActionFrontControllerSetMedia()
    {
        if (empty($this->context->controller->page_name) || $this->context->controller->page_name !== 'checkout') {
            return;
        }
        Media::addJsDef([
            'sebpaymentgateway_checkout_params' => [
                'sebpaymentgateway_delete_txt' => $this->l('Are you sure you want to delete the card ?'),
                'retry_redirect' => $this->context->link->getPageLink('order', true, NULL)
                    . '?submitReorder=&id_order=ORDER_REFERENCE',
            ],
        ]);
        $this->context->controller->registerJavascript(
            'sebpaymentgateway-script',
            'modules/' . $this->name . '/views/js/sebpaymentgateway.js'
        );
    }

    public function getFormattedPaymentMethod ($method = 'missing_payment_method') {
        // usage in controller:
        // $this->module->getFormattedPaymentMethod (Tools::getValue('payment_method') ? Tools::getValue('payment_method') : 'missing_payment_method'),

        $output = 'SEB Payment Gateway';
        switch ($method) {
            case 'apple_pay':
            case 'card' :
                $output = $this->l('Card payment', 'validation');
                break;
            case 'paypal' :
                $output = $this->l('Paypal payment', 'validation');
                break;
            case 'missing_payment_method':
            case '' :
                // these 2 provide module name as output
                break;
            default :
                // this covers various bank payment methods, eg: swed_ob_ee
                $output = $this->l('Bank payment', 'validation');
                break;
        }

        return $output;
    }

    protected function getEstonianLangId()
    {
        $sql = new DbQuery();
        $sql->select('id_lang');
        $sql->from('lang');
        $sql->where("iso_code = 'et'");
        $sql->where("active = 1");
        $results = Db::getInstance()->getValue($sql);
        return $results;
    }

    public function logError($errorObj, $identifier = '')
    {
        $log = date('d-m-Y H:i:s').' : ' . $identifier;
        $log .= "\n";
        if (!empty($errorObj)) {
            $errorText = print_r( $errorObj, true);
            $log .= $errorText;
            $log .= "\n";
        }

        $file = dirname(__FILE__).'/log/debug.log';

        file_put_contents($file, $log, FILE_APPEND | LOCK_EX);
    }

    public function insertOrderMessage ($msgText, $order)
    {
        $customer = new Customer ($order->id_customer);

        $ct = new CustomerThread();
        $ct->id_contact = 0;
        $ct->id_customer = (int) $order->id_customer;
        $ct->id_shop = (int) $order->id_shop;
        $ct->id_order = (int) $order->id;
        $ct->id_lang = (int) $order->id_lang;
        $ct->email = $customer->email;
        $ct->status = 'closed';
        $ct->token = Tools::passwdGen(12);
        $ct->add();

        $cm = new CustomerMessage();
        $cm->id_customer_thread = $ct->id;
        $cm->message = $msgText;
        $cm->private = 1;
        $cm->add();
    }

}
