/*
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    SEBPAYMENTGATEWAY
 *  @copyright since 2010 SEBPAYMENTGATEWAY
 *  @license   LICENSE.txt
 */
"use strict";

var SEBPAYMENTGATEWAY = {};
SEBPAYMENTGATEWAY.init = function() {
    SEBPAYMENTGATEWAY.hideDefaultSortOrder();
    SEBPAYMENTGATEWAY.addSortableId();
    SEBPAYMENTGATEWAY.sortElements();
    SEBPAYMENTGATEWAY.buildSortableList();
    SEBPAYMENTGATEWAY.attachButtonEvent();

    new Sortable(document.querySelector('#sortable'), {
        animation: 150,
    });
}


SEBPAYMENTGATEWAY.attachButtonEvent = function() {
    const button = document.querySelector('button[name="submitAddconfiguration"]');
    button.addEventListener('click', function() {
        SEBPAYMENTGATEWAY.updateSortOrder();
    });
}

/**
 * Get the Methods Order input.
 *
 * @returns {*}
 */
SEBPAYMENTGATEWAY.getMethodsOrderInput = function() {
    return document.getElementById('methods_order');
}

/**
 * Change te Methods Order input's type to hidden.
 */
SEBPAYMENTGATEWAY.hideDefaultSortOrder = function() {
    SEBPAYMENTGATEWAY.getMethodsOrderInput().type = 'hidden';
    SEBPAYMENTGATEWAY.getMethodsOrderInput().parentNode.parentNode.style.display = "none";
}

/**
 * Add the 'sortable' id to the checkboxes parent div.
 */
SEBPAYMENTGATEWAY.addSortableId = function() {
    const checkboxLabel = document.querySelector('label[for^="enabled_payment_option_"]');
    checkboxLabel.parentNode.parentNode.setAttribute('id', 'sortable');
}

/**
 * Build the sortable list based on the available payment types.
 */
SEBPAYMENTGATEWAY.buildSortableList = function() {
    const checkboxLabels = document.querySelectorAll('label[for^="enabled_payment_option_"]');

    for (let label of checkboxLabels) {
        const childSpan = document.createElement('span');
        childSpan.classList.add('ui-icon', 'ui-icon-arrowthick-2-n-s');

        const span = document.createElement('span');
        span.classList.add('method-order', 'ui-state-default');
        span.textContent = label.textContent;

        SEBPAYMENTGATEWAY.removeTextContent(label);

        span.prepend(childSpan);
        label.appendChild(span);
    }
}

SEBPAYMENTGATEWAY.removeTextContent = function(element) {
    let childNodes = element.childNodes;
    childNodes.forEach(c => c.nodeType === Node.TEXT_NODE && c.remove());
}

/**
 * Get the available payment types and sort them based on the Methods Order input.
 */
SEBPAYMENTGATEWAY.sortElements = function() {
    const parent = document.querySelector('#sortable');
    const checkboxDivs = parent.querySelectorAll('.checkbox');
    const sortOrder = SEBPAYMENTGATEWAY.getSortOrder();

    Array.prototype.slice.call(checkboxDivs).sort(function(a, b) {
        const aIndex = a.querySelector('label').htmlFor.replace('enabled_payment_option_', '');
        const bIndex = b.querySelector('label').htmlFor.replace('enabled_payment_option_', '');

        return sortOrder.indexOf(aIndex) - sortOrder.indexOf(bIndex);
    }).forEach(function(el) {
        parent.appendChild(el);
    });

    parent.appendChild(parent.querySelector('p'));
}

/**
 * Get the sort order from the hidden input.
 *
 * @returns {*}
 */
SEBPAYMENTGATEWAY.getSortOrder = function() {
    return SEBPAYMENTGATEWAY.getMethodsOrderInput().value.split(',');
}

/**
 * Update the sort order input based on the current sort order.
 */
SEBPAYMENTGATEWAY.updateSortOrder = function() {
    const methodsOrder = document.querySelectorAll('#sortable label[for^="enabled_payment_option_"]');
    let order = '';
    for (let i = 0; i < methodsOrder.length; i++) {
        order += methodsOrder[i].htmlFor.replace('enabled_payment_option_', '')

        if (i !== methodsOrder.length - 1) {
            order += ',';
        }
    }

    SEBPAYMENTGATEWAY.getMethodsOrderInput().value = order;
}

$(document).ready(function() {
    SEBPAYMENTGATEWAY.init();
})
